<template>
  <div :id="id">
    <template v-if="field.displayType == Constant.PermissionType_HIDDEN || field.textType == 'hidden'">
      <!--隐藏-->
      <input type="hidden" v-model="field.value" />
      <!-- <span style="width:100%;padding-left:10px;">{{field.hiddenValue}}</span> -->
      <a class="mint-cell mint-field" v-if="field.mobile&&field.hiddenValue">
        <div class="mint-cell-wrapper">
          <div class="mint-cell-title">
            <span class="mint-cell-text">{{field.discript?field.discript:field.name}}</span> 
          </div> 
          <div class="mint-cell-value readOnly">
            <span>{{field.hiddenValue}}</span>
          </div> 
        </div>
      </a>
    </template>

    <template v-else-if="field.textType == 'readonly' || field.displayType == Constant.PermissionType_READONLY || field.displayType == Constant.PermissionType_DISABLED">
      <div class="records" v-if="field.mobile">
        <div class="records-con">
          <div class="left-of-record">
            {{field.discript?field.discript:field.name}}
          </div>
          <!-- @touchend="stopRecord" -->
          <div class="right-of-record" v-show="!field.value">
            点击说话
          </div> 
          <div class="sound" v-show="field.value">
            <div class="readonly-sound" @touchstart='play(recordUrl)'>
              <i class="fa fa-volume-up icon-v"></i>
              {{time}}''
            </div>
          </div>
        </div>
      </div>
      <audio v-if="field.audioUrl" :src="field.audioUrl" ref="au1"></audio>
    </template>

    <template v-else-if="field.displayType == Constant.PermissionType_MODIFY">
      <!--可编辑-->
      <div class="records"  v-if="field.mobile">
        <div class="records-con">
          <div class="left-of-record">
            {{field.discript?field.discript:field.name}}
          </div>
          <!-- @touchend="stopRecord" -->
          <div class="right-of-record" v-if="!field.value" @click="startRecord">
            {{$t('speak')}}
          </div>
          <div class="sound" v-if="field.value">
            <div class="sound-left" @click='play(field.recordUrl?field.recordUrl:recordUrl)'>
              <i class="fa fa-volume-up icon-v"></i>
              {{time?time:field.time}}''
            </div>
            <div class="sound-right" @click.stop.prevent="deleteRecord(field.value)">
              <i class="fa fa-times-circle fa-lg"></i>
            </div>
          </div>
        </div>
      </div>
      <audio id="audios" :src="field.audioUrl" ref="au"></audio>
      <audio  v-show="isShow" @canplay="getDuration" @timeupdate="updateTime" id="aud" ref="aud" controls>
        <source :src="field.audioUrl" type="audio/ogg">
        <source :src="field.audioUrl" type="audio/mpeg">
        {{$t('noSupport')}}
      </audio>
    </template>
    <comm-dialog
      v-if="dialogVisible"
      :visible.sync="dialogVisible"
      :lock-scroll="true"
      position="bottom"
    >
      <div class="records-detail">
        <div>{{recordTime}}</div>
        <div class="microphone">
          <i class="fa fa-microphone fa-2x"></i>
        </div>
        <div class="time">{{$t('stop60recording')}}</div>
        <div class="record-btn">
          <span class="icon-stop" @touchstart="stopRecord">
            <i class="fa fa-stop-circle-o fa-lg"></i>
          </span>
          <span class="stop">
            {{$t('stop')}}
          </span>
        </div>
      </div>
    </comm-dialog>
  </div>
</template>

<script>
import wx from "weixin-js-sdk";
import Constant from "@/Constant.js";
import controlHidden from "@/assets/js/controlHidden.js"
export default {
  name: "o-record",
  props: ["id"],
  watch: {
    dialogVisible(val) {
      if(!val) {
        this.stopRecord(); 
      } 
    },

    tap () {
      if(this.field.audioUrl) {
        this.$refs.au.load();
        this.$refs.au.oncanplay = () => {      
          // this.time = Math.floor(this.$refs.au.duration);
          this.recordedTime = this.time;
          this.audio = new Audio();
          this.audio.src = this.field.audioUrl;
        }
      }
    }
  },

  computed: {
    field: function() {
      let formData = this.$parent.findField(this.id)
      if(formData.value && formData.value.length>0) {
        
        formData.value = JSON.parse(formData.value);
        this.time = parseInt(formData.value[0].durations);
       
        let host = 'http://' + window.location.host + '/obpm';
        formData.audioUrl = host + formData.value[0].path;
        
      }
      return this.$parent.findField(this.id);
    }
  },
  mounted(){
    window['onClickRecord'] = (res)=> {
      this.onClickRecord(res);
    }
    this.$nextTick(()=> {
      this.tap = !this.tap;
      let field = this.field;
      if(field.isHideType) {
        let id = this.id,
          formShowType = field.formShowType;
        controlHidden({id, field, formShowType})
      }
    });
  },
  data: function() {
    return {
      Constant,
      isRecord:true,
      voices:'',
      pathData:'',
      dialogVisible:false,
      uploadData:'',
      timer:'',
      time:0,
      recordTime:'00:00',
      recordUrl:'',
      isPlay:true,
      playRecord:'',
      recordedTime:'',
      tap: false,
      newData:'',
      isShow:false,
      ceshi2:''
    }
  },

  methods: {
    onClickRecord(res){
      let that = this;
      let datas = JSON.parse(res);
      let currentfieldId = localStorage.getItem("currentRecordFieldId");
      let list = this.$parent.getAllFields();
      let uploadData = datas.path;
      let arr = JSON.stringify([{
        durations:datas["len"],
        path:uploadData,
      }])

      let host = 'http://' + window.location.host + '/obpm';
      let audioUrl = host + uploadData;

      for(let i=0; i<list.length; i++) {
        if(currentfieldId == list[i].id) {
          list[i].recordedTime = datas["len"];
          list[i].value = arr;
          list[i].time = datas["len"];
          list[i].audioUrl = audioUrl;
          list[i].recordUrl = audioUrl;
        }
      }
      that.audio = new Audio();
      that.audio.src = that.recordUrl;
    },

    getDuration() {
      setTimeout(() => {
        console.log(this.$refs.aud.duration);
      }, 800)
       //此时可以获取到duration
    },
    updateTime(e) {
      this.currentTime = e.target.currentTime;  //获取audio当前播放时间
    },

    //播放录音
    play() {
      let ua = window.navigator.userAgent;
      if(ua.indexOf("myApps") >= 0) { //嗨办公
        let field = this.field;
        this.$refs.aud.load();
        if(this.isPlay) {
          this.isPlay = false;
          this.$refs.aud.play();
          let playTime = field.time?field.time:parseInt(this.time);
          this.playRecord = setInterval(() => {
            playTime--;
            if(playTime < 10) {
              this.time = "0" + playTime;
            }else {
              this.time = playTime;
            }
            if(playTime == 0) {
              this.time = this.recordedTime;
              this.isPlay = true;
              clearInterval(this.playRecord);
            }
          }, 1000)
        }else if (!this.isPlay) {
          this.isPlay = true;
          this.$refs.aud.pause();
          clearInterval(this.playRecord);
        }
      }else {
        if(this.isPlay) {
          this.$refs.au.load();
          // this.getDuration()
          // this.time = Math.floor(this.$refs.au.duration);
          this.isPlay = false;
          this.audio.play();
          let playTime = parseInt(this.time);
          this.playRecord = setInterval(() => {
            playTime--;
            if(playTime < 10) {
              this.time = "0" + playTime;
            }else {
              this.time = playTime;
            }
            if(playTime == 0) {
              this.time = this.recordedTime;
              this.isPlay = true;
              clearInterval(this.playRecord);
            }
          }, 1000)
        }else if (!this.isPlay) {
          this.isPlay = true;
          this.audio.pause();
          clearInterval(this.playRecord);
        }
      }
    },

    startRecord() {
      localStorage.setItem('currentRecordFieldId',this.field.id);
      let ua = window.navigator.userAgent;
      if(ua.indexOf("myApps") >= 0) { //嗨办公
        this.getRecord({
          result: (res) => {
            onClickRecord(res);
            // let datas = res;
            // that.newData = datas;
            // that.time = datas["len"];
            // that.recordedTime = that.time;
            // that.uploadData = datas.path;
            // // that.uploadData = that.uploadData.replace(".amr",".mp3");
            // let arr = JSON.stringify([{
            //   durations:that.time,
            //   path:that.uploadData,
            // }])
            // that.field.value = arr;
            // let host = 'http://' + window.location.host + '/obpm';
            // that.recordUrl = host + that.uploadData;
            // that.audio = new Audio();
            // that.audio.src = that.recordUrl;
          }
        });
      }else {
        let _url = location.href.split("#")[0];
        this.$api.getWeixinApi(
          encodeURIComponent(_url),
          {  
            onSucess: response => {
              let that = this;
              wx.config({
                debug: false, // 开启调试模式,调用的所有api的返回值会在客户端alert出来，若要查看传入的参数，可以在pc端打开，参数信息会通过log打出，仅在pc端时才会打印。
                appId: response.data.data.appId, // 必填，公众号的唯一标识
                timestamp: response.data.data.timestamp, // 必填，生成签名的时间戳
                nonceStr: response.data.data.nonceStr, // 必填，生成签名的随机串
                signature: response.data.data.signature, // 必填，签名，见附录1
                jsApiList: [
                  "checkJsApi",
                  "onMenuShareTimeline",
                  "onMenuShareAppMessage",
                  "onMenuShareQQ",
                  "onMenuShareWeibo",
                  "hideMenuItems",
                  "showMenuItems",
                  "hideAllNonBaseMenuItem",
                  "showAllNonBaseMenuItem",
                  "translateVoice",
                  "startRecord",
                  "stopRecord",
                  "onRecordEnd",
                  "playVoice",
                  "pauseVoice",
                  "stopVoice",
                  "uploadVoice",
                  "downloadVoice",
                  "chooseImage",
                  "previewImage",
                  "uploadImage",
                  "downloadImage",
                  "getNetworkType",
                  "openLocation",
                  "getLocation",
                  "hideOptionMenu",
                  "showOptionMenu",
                  "closeWindow",
                  "scanQRCode",
                  "chooseWXPay",
                  "openProductSpecificView",
                  "addCard",
                  "chooseCard",
                  "openCard"
                ] // 必填，需要使用的JS接口列表，所有JS接口列表见附录2
              });
              wx.startRecord({
                success: function(){
                  localStorage.rainAllowRecord = 'true'
                  that.dialogVisible = true;
                  that.time = 0;
                  that.timer = setInterval(() => {
                    that.time++
                    that.recordedTime = that.time;
                    if(that.time >= 60) {
                      clearInterval(that.timer)
                      that.stopRecord()
                    }
                    if(that.time <10) {
                      that.recordTime = "00:0"+that.time;
                    }else if (that.time < 60) {
                      that.recordTime = "00:"+that.time;
                    }else if(that.time >= 60) {
                      that.recordTime = "01:"+"00";
                    }
                  }, 1000)
                  // that.oRecordInfo.useWxRecord = 3
                  //that.oRecordInfo.bShowRecording = true  //  控制正在录音gif显示
                },
                cancel: function () {
                
                }
              })

            }
          }
        );
      }
    },

    getRecord(callback) {
      let that = this;
      let u = navigator.userAgent;
      let isiOS = !!u.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/);
      let isAndroid = u.indexOf('Android') > -1 || u.indexOf('Adr') > -1; //android终端
      if(isiOS){
        let result = callback.result;
        callback.result = result.toString();
        window.webkit.messageHandlers.startRecord.postMessage(callback);
      }else if(isAndroid){
        let _native = top.nativeAPI ? top.nativeAPI : nativeAPI;
        _native.startRecord({
            success: function (res) {
              var datas = JSON.parse(res);
              that.newData = datas;
              that.time = datas["len"];
              that.recordedTime = that.time;
              that.uploadData = datas.path;
              that.uploadData = that.uploadData.replace(".amr",".mp3");
              let arr = JSON.stringify([{
                durations:that.time,
                path:that.uploadData,
              }])
              that.field.value = arr;
              let host = 'http://' + window.location.host + '/obpm';
              that.recordUrl = host + that.uploadData;
              that.audio = new Audio();
              that.audio.src = that.recordUrl;
            }
        });
    
      }

    },

    //停止录音
    stopRecord() {
      clearInterval(this.timer)
      this.dialogVisible = false;
      let that = this;
      wx.stopRecord({
        success: function (res) {
          that.voices = res.localId;
          that.uploadVoice(res.localId);
        },
        fail: function (res) {
          alert(JSON.stringify(res));
        }
      });
      //this.isRecord = true;
    },

    //微信API获取serverId
    uploadVoice(localId) {
      let that = this;
      wx.uploadVoice({
        localId: localId, // 需要上传的音频的本地ID，由stopRecord接口获得
        isShowProgressTips: 1,// 默认为1，显示进度提示
        success: function (res) {
          that.pathData = res.serverId;
          that.uploadFile(that.pathData);
        },
        fail: function (res) {
          alert(JSON.stringify(res));
        }
      });
    },

    //上传文件
    uploadFile(id) {
      this.$api.uploadWeixinRecord(id,{
          onSucess: (res) => {
            this.uploadData = res.data.data
            this.uploadData = this.uploadData.replace(".amr",".mp3");
          
            let arr = JSON.stringify([{
              durations:this.time,
              path:this.uploadData,
            }])

            //this.field.value = this.uploadData;
            this.field.value = arr;
            let host = 'http://' + window.location.host + '/obpm';
            this.recordUrl = host + this.uploadData;
            this.audio = new Audio();
            this.audio.src = this.recordUrl;
          }
        }
      )
    },

    //删除录音
    deleteRecord(recordUrl) {
      let appId = this.$root.appid;
      let formId = this.field.formId;
      let docId = this.field.docId;
      let path = recordUrl.path;
      this.$api.deleteUpload(appId, formId, docId, path, {
        onSucess: (res) => {
          if(res.data.errcode == 0) {
            this.field.value = ''; 
          }
        }
      });
    },
  },

  
}
</script>

<style lang="scss">
  .records {
    width:100%;
    height:48px;
    border-bottom:1px solid #efeffa;
    .records-con {
      width:100%;
      height: 48px;
      padding: 0 10px;
      box-sizing: border-box;
      display: -webkit-flex;
      display: flex;
      justify-content: space-between;
      align-items: center;
      .left-of-record {
        width:100px;
        overflow: hidden;
        text-overflow: ellipsis;
        white-space: nowrap;
        font-size:14px;
        font-family:PingFangSC-Regular,PingFang SC;
        
      }
      .right-of-record {
          width:250px;
          height: 35px;
          line-height: 35px;
          background: #FAFAFA;
          border:1px solid #ccc;
          border-radius: 5px;
          text-align:center;
          font-size: 14px;
      }
      .sound {
        width:250px;
        height: 35px;
        display: -webkit-flex;
        display: flex;
        flex-direction: row;
        .sound-left {
          display: -webkit-flex;
          display: flex;
          width:225px;
          height: 35px;
          align-items: center;
          font-size: 14px;
          background: #26A2FF;
          color:white;
          border-radius: 5px;
          .icon-v {
            margin-left:12px;
            margin-right:10px;
          }
        }
        .sound-right {
          width:25px;
          height: 35px;
          display: -webkit-flex;
          display: flex;
          justify-content: flex-end;
          align-items: center;
          font-size: 12px;
          color:#888888;
        }
      }
      .readonly-sound {
        display: -webkit-flex;
        display: flex;
        width:250px;
        height: 35px;
        align-items: center;
        font-size: 14px;
        background: #26A2FF;
        color:white;
        border-radius: 5px;
        .icon-v {
          margin-left:12px;
          margin-right:10px;
        }
      }
    }
  }
  .records-detail {
    width:100%;
    height:270px;
    position: absolute;
    bottom: -30px;
    left: 0;
    background: white;
    z-index: 6666;
    padding: 10px 0;
    box-sizing: border-box;
    &>div {
      padding: 10px 0;
      box-sizing: border-box;
      text-align: center;
    }
    .microphone {
      color: #888888;
      font-size: 20px;
    }
    .time {
      font-size: 12px;
      color:#26A2FF;
    }
    .record-btn {
      display: -webkit-flex;
      display: flex;
      flex-direction: column;
      &>span {
        display: inline-block;
        margin-bottom: 8px;
        color: #666666;
      }
      .icon-stop {
        font-size: 18px;
        color: #888888;
      }
      .stop {
        font-size: 12px;
      }
    }
  }
</style>